package de.hhu.ba.yoshikoWrapper.core;
//TODO: ADD LOGGER SYSTEM

import org.cytoscape.model.CyColumn;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskMonitor;
import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.gui.ClusterView;
import de.hhu.ba.yoshikoWrapper.gui.SolutionTab;
import de.hhu.ba.yoshikoWrapper.gui.SolutionsPanel;
import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_std__vectorT_int_t;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_yskInput__LibraryInput;
import de.hhu.ba.yoshikoWrapper.swig.SWIGTYPE_p_ysk__ClusterEditingSolutions;

public class AlgorithmTask extends AbstractTask {
	
	
	//Symbolic links
	private static Logger logger = YoshikoLogger.getInstance().getLogger();

	//Parameters
	private double insertionCostDefault;
	private double deletionCostDefault;
	private CyColumn permanentColumn;
	private CyColumn forbiddenColumn;
	private CyColumn weightColumn;
	private int timeLimit;
	private CyNetwork net;
	private String bitMaskRules;
	private SolutionsPanel solutionsPanel;

	public AlgorithmTask(
			CyNetwork net,
			int timeLimit,
			CyColumn weightColumn,
			CyColumn permanentColumn,
			CyColumn forbiddenColumn,
			double insertionCostDefault,
			double deletionCostDefault,
			String bitMaskRules,
			double multiplicativeFactor,
			boolean separatePartitionCuts,
			boolean separateTriangles,
			boolean useHeuristic,
			SolutionsPanel solutionsPanel
			) 
	{
		this.net = net;
		this.timeLimit = timeLimit;
		this.weightColumn = weightColumn;
		this.permanentColumn = permanentColumn;
		this.forbiddenColumn = forbiddenColumn;
		this.insertionCostDefault = insertionCostDefault;
		this.deletionCostDefault = deletionCostDefault;
		this.bitMaskRules = bitMaskRules;
		this.multiplicativeFactor = multiplicativeFactor;
		this.separatePartitionCuts = separatePartitionCuts;
		this.separateTriangles =separateTriangle;
		this.useHeuristic = useHeuristic;
		this.solutionsPanel = solutionsPanel;
	}

	@Override
	public void run(TaskMonitor taskMonitor) throws Exception {
		taskMonitor.setTitle(LocalizationManager.get("yoshTask"));
		taskMonitor.setProgress(0.0);
		//Get current network
		if (net == null) {
			logger.warn("CoreAlgorithm called on a net that is NULL!");
			return;
		}
		
		//Set time limit
		LibraryInterface.setTimeLimit(timeLimit);

		//Create a node map to identify nodes and eges in the solution
		NodeMap nodeMap  = new NodeMap(net);
		taskMonitor.setProgress(0.1);

		//Generate an input instance from the network
		SWIGTYPE_p_yskInput__LibraryInput input = NetworkParser.parseNetwork(
				net,
				nodeMap,
				weightColumn,
				permanentColumn, 
				forbiddenColumn,
				deletionCostDefault
				);
		taskMonitor.setProgress(0.2);

		//Set the default value for insertion cost
		LibraryInterface.LibraryInput_setDefaultInsertionCost(input, insertionCostDefault);

		//Call Yoshiko <<< Algorithm is performed here
		SWIGTYPE_p_ysk__ClusterEditingSolutions solutions = LibraryInterface.processLibraryInput(
				input,
				1,
				bitMaskRules,
				multiplicativeFactor, 
				separatePartitionCuts,
				separateTriangles,
				useHeuristic
				);
		taskMonitor.setProgress(0.9);

		this.solutionsPanel.reset();

		
		long numberOfSolutions = LibraryInterface.ClusterEditingSolutions_getNumberOfSolutions(solutions);
		taskMonitor.setStatusMessage("Found: "+numberOfSolutions+" solutions!");
		
		double modificationCost = LibraryInterface.ClusterEditingSolutions_getTotalCost(solutions);
		taskMonitor.setStatusMessage("Paid a total modification cost of: "+modificationCost);
		
		for (long i=0;i<numberOfSolutions;i++) {
			
			taskMonitor.setStatusMessage("Processing solution "+(i+1)+" of "+numberOfSolutions);
		
			SolutionTab tab = this.solutionsPanel.addSolutionTab(i);
			String columnName = "YOSHIKO_SOLUTION_"+(i+1);
			
			net.getDefaultNodeTable().deleteColumn(columnName);
			net.getDefaultNodeTable().createColumn(columnName, String.class, false);
			
			long numberOfClusters = LibraryInterface.ClusterEditingSolutions_getNumberOfClusters(solutions, i);
			
			for (long k=0;k<numberOfClusters;k++) {
				ClusterView cV = tab.addCluster(k);
				cV.setTitle((k+1));
				taskMonitor.setStatusMessage("Processing cluster "+(k+1)+" of "+numberOfClusters);
				SWIGTYPE_p_std__vectorT_int_t cluster = LibraryInterface.ClusterEditingSolutions_getCluster(solutions, i, k);
				long sizeOfCluster = LibraryInterface.IntVector_size(cluster);
				cV.setClusterSize(sizeOfCluster);
				for (int l=0;l<sizeOfCluster;l++) { //Unsafe mismatch int long
					taskMonitor.setStatusMessage("Processing entry "+(l+1)+ " of "+sizeOfCluster);
					int nodeID = LibraryInterface.IntVector_get(cluster, l);
					CyNode node = nodeMap.indexOf(nodeID); //<<< Another int/long conversion
					cV.addNode(node);
					if (node == null) {
						logger.error("FATAL ERROR: Node was not previously mapped!");
						return;
					}
					net.getRow(node).set(columnName, ""+(k+1));
				}
			}
		}

		solutionsPanel.revalidate();
		solutionsPanel.repaint();
		
		LibraryInterface.delete_LibraryInput(input);		
		taskMonitor.setProgress(1.0);
	}
}
